@@ -3,7 +3,7 @@ package Make;
 use strict;
 use warnings;
 
-our $VERSION = '2.002';
+our $VERSION = '2.003';
 
 use Carp qw(confess croak);
 use Config;
@@ -24,7 +24,7 @@ my %date;
 my %fs_function_map = (
     glob          => sub { glob $_[0] },
     fh_open       => sub { open my $fh, $_[0], $_[1] or die "open @_: $!"; $fh },
-    fh_write      => sub { my $fh = shift; print {$fh} @_ },
+    fh_write      => sub { my $fh = shift;                                 print {$fh} @_ },
     file_readable => sub { -r $_[0] },
     mtime         => sub { ( stat $_[0] )[9] },
 );
@@ -91,6 +91,11 @@ sub patmatch {
     return;
 }
 
+sub in_dir {
+    my ( $file, $dir ) = @_;
+    defined $dir ? "$dir/$file" : $file;
+}
+
 sub locate {
     my ( $self, $file ) = @_;
     my $readable = $self->fsmap->{file_readable};
@@ -98,7 +103,7 @@ sub locate {
         next unless defined( my $Pat = patmatch( $key, $file ) );
         foreach my $dir ( @{ $self->{Vpath}{$key} } ) {
             ( my $maybe_file = $dir ) =~ s/%/$Pat/g;
-            return $maybe_file if $readable->($maybe_file);
+            return $maybe_file if $readable->( in_dir $maybe_file, $self->{InDir} );
         }
     }
     return;
@@ -115,10 +120,7 @@ sub dotrules {
             next unless my $r = delete $Dot->{ $f . $t };
             DEBUG and print STDERR "Pattern %$t : %$f\n";
             my $target   = $self->target( '%' . $t );
-            my @dotrules = @{ $r->rules };
-            die "Failed on pattern rule for '$f$t', too many rules"
-                if @dotrules != 1;
-            my $thisrule = $dotrules[0];
+            my $thisrule = $r->rules->[-1];             # last-specified
             die "Failed on pattern rule for '$f$t', no prereqs allowed"
                 if @{ $thisrule->prereqs };
             my $rule = Make::Rule->new( '::', [ '%' . $f ], $thisrule->recipe );
@@ -134,7 +136,7 @@ sub dotrules {
 sub date {
     my ( $self, $name ) = @_;
     unless ( exists $date{$name} ) {
-        $date{$name} = $self->fsmap->{mtime}->($name);
+        $date{$name} = $self->fsmap->{mtime}->( in_dir $name, $self->{InDir} );
     }
     return $date{$name};
 }
@@ -251,7 +253,7 @@ sub tokenize {
     my ( $string, @extrasep ) = @_;
     ## no critic ( BuiltinFunctions::RequireBlockGrep BuiltinFunctions::RequireBlockMap)
     my $pat  = join '|', '\s+', map quotemeta, @extrasep;
-    my @toks = grep length, parse_line $pat, 1, $string;
+    my @toks = grep defined && length, parse_line $pat, 1, $string;
     ## use critic
     s/\\(\s)/$1/g for @toks;
     return \@toks;
@@ -306,6 +308,7 @@ sub process_ast_bit {
         my ($tokens) = tokenize( $self->expand( $args[1] ) );
         foreach my $file (@$tokens) {
             eval {
+                $file = in_dir $file, $self->{InDir};
                 my $mf  = $self->fsmap->{fh_open}->( '<', $file );
                 my $ast = parse_makefile($mf);
                 close($mf);
@@ -414,9 +417,13 @@ sub pseudos {
 }
 
 sub find_makefile {
-    my ( $file, $extra_names, $fsmap ) = @_;
-    return $file if defined $file;
-    for ( qw(makefile Makefile), @{ $extra_names || [] } ) {
+    my ( $file, $extra_names, $fsmap, $dir ) = @_;
+    return in_dir $file, $dir if defined $file;
+    my @search = ( qw(makefile Makefile), @{ $extra_names || [] } );
+    ## no critic (BuiltinFunctions::RequireBlockMap)
+    @search = map in_dir( $_, $dir ), @search;
+    ## use critic
+    for (@search) {
         return $_ if $fsmap->{file_readable}->($_);
     }
     return;
@@ -424,7 +431,7 @@ sub find_makefile {
 
 sub parse {
     my ( $self, $file ) = @_;
-    $file = find_makefile $file, $self->{GNU} ? ['GNUmakefile'] : [], $self->fsmap;
+    $file = find_makefile $file, $self->{GNU} ? ['GNUmakefile'] : [], $self->fsmap, $self->{InDir};
     my $fh;
     if ( ref $file eq 'SCALAR' ) {
         open my $tfh, "+<", $file;
@@ -668,6 +675,11 @@ Defaults to code accessing the actual local filesystem. The various
 functions are expected to return real Perl filehandles. Relevant keys:
 C<glob>, C<fh_open>, C<fh_write>, C<mtime>.
 
+=head3 InDir
+
+Optional. If supplied, will be treated as the current directory instead
+of the default which is the real current directory.
+
 =head2 parse
 
 Parses the given makefile. If none or C<undef>, these files will be tried,
